#include <Arduino.h>
#include <Servo.h>
#include <Ultrasonic.h>

#include <Wire.h>
#include <SPI.h>
// #include "Adafruit_TCS34725.h"
 
// Инициализация модуля с установками времени интеграции и усиления
// Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X);
 

Ultrasonic ultrasonic_sensor(12, 13);

Servo servo;
const int servo_pin = 9;

const int line_sensor_left = A0;
const int line_sensor_right = A1;

const int pinH1 = 7;  // Создаём константу указывая номер вывода H1 MotorShield (он управляет направлением 1 мотора)
const int pinE1 = 6;  // Создаём константу указывая номер вывода E1 MotorShield (он управляет скоростью    1 мотора)
const int pinE2 = 5;  // Создаём константу указывая номер вывода E2 MotorShield (он управляет скоростью    2 мотора)
const int pinH2 = 4;  // Создаём константу указывая номер вывода H2 MotorShield (он управляет направлением 2 мотора)

const int IR_sensor = A5;

const int null_deg_pos = 150;
const int up_deg_pos = 90;
const int down_deg_pos = 30;

unsigned long old_millis = 0;

const int threshold_level = 980;
const int threshold_level_1 = 940;

int delays = 100;

void stop() {
  digitalWrite(pinH1, 0);
  digitalWrite(pinH2, 0);

  analogWrite(pinE1, 0);
  analogWrite(pinE2, 0);
}

// нулевое - 180 нижнее - 25, верхнее - 70

void drive(int speed_1, int speed_2) {

  bool direct_1 = false;
  bool direct_2 = false;

  if (speed_1 > 0) direct_1 = true;
  if (speed_2 > 0) direct_2 = true;

  digitalWrite(pinH1, direct_1);
  digitalWrite(pinH2, direct_2);

  analogWrite(pinE1, abs(speed_1)); // 0.92
  analogWrite(pinE2, abs(speed_2)*0.86);

}


void drive_time(int speed, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, speed);
  }
  stop();
}

void drive_time_2(int speed1, int speed2, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed1, speed2);
  }
  stop();
}


void line(int speed = 40, int speed_slow = 0, int delay_=5) {

  bool line_sensor_left_value = false;
  bool line_sensor_right_value = false;

  if (analogRead(line_sensor_left) < threshold_level) line_sensor_left_value = true;
  if (analogRead(line_sensor_right) < threshold_level) line_sensor_right_value = true;

  Serial.print(line_sensor_left_value);
  Serial.print(" ");
  Serial.println(line_sensor_right_value);

  if (line_sensor_left_value == 0 && line_sensor_right_value == 0)
    drive(speed, speed);
  else if (line_sensor_left_value == 1 && line_sensor_right_value == 0) {
    drive(speed_slow, speed);
    delay(delay_);
  }
  else if (line_sensor_left_value == 0 && line_sensor_right_value == 1) {
    drive(speed, speed_slow);
    delay(delay_);
  } else {
    drive(speed, speed);
  }
}

void line_time(int speed, int speed_slow, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      line(speed, speed_slow);
  }
  stop();
}

void line_per(int speed, int speed_slow, int n) {
  for (int i = 0; i < n; i++) {
    for (;;) {
      if (analogRead(line_sensor_right) < threshold_level and analogRead(line_sensor_left) < threshold_level) {
        stop();
        break;
      }
      line(speed, speed_slow, 5);
    }
    delay(100);
    stop();
  }
}

void line_distance_sensor(int speed = 40, int speed_slow = 0, int dist = 40) {
  for (;;) {
    if (ultrasonic_sensor.read() < dist) {
      stop();
      break;
    }
    line(speed, speed_slow);
    delay(5);
  }
}



void turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    // while ((analogRead(line_sensor_left) < 850 && speed < 0) || (analogRead(line_sensor_right) < 850 && speed > 0)) {
    //   drive(speed, -speed);
    // }
    // delay(10);

    while ((analogRead(line_sensor_left) > threshold_level_1 && speed < 0) || (analogRead(line_sensor_right) > threshold_level_1 && speed > 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void anti_turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    // while ((analogRead(line_sensor_left) < 850 && speed < 0) || (analogRead(line_sensor_right) < 850 && speed > 0)) {
    //   drive(speed, -speed);
    // }
    // delay(10);

    while ((analogRead(line_sensor_left) > threshold_level && speed > 0) || (analogRead(line_sensor_right) > threshold_level && speed < 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void both_turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    // while ((analogRead(line_sensor_left) < 850 && speed < 0) || (analogRead(line_sensor_right) < 850 && speed > 0)) {
    //   drive(speed, -speed);
    // }
    // delay(10);

    while (analogRead(line_sensor_left) > threshold_level || analogRead(line_sensor_right) > threshold_level && speed > 0) {
      drive(speed, -speed);
    }
    stop();
  }
}

void turn_t(int speed, int time_) {
  unsigned long millis_old = millis();
  while (millis() - millis_old < time_) {
    drive(speed, -speed);
  }
  stop();
}


// int getLuxSensor(){
//   uint16_t r, g, b, c, colorTemp, lux;
 
//   // Получение данных о цвете с модуля
//   tcs.getRawData(&r, &g, &b, &c);
  
//   // Расчет температуры цвета и уровня освещенности
//   colorTemp = tcs.calculateColorTemperature(r, g, b);
//   lux = tcs.calculateLux(r, g, b);
  
//   return lux;
// }


void setup() {
  Serial.begin(9600);

  // Проверка наличия модуля TCS34725
  // if (tcs.begin()) {
  //     Serial.println("Дачкий найден!"); // Если модуль найден, вывод сообщения
  // } else {
  //     Serial.println("Дачик TCS34725 не найден ... проверьте соединение"); // Если модуль не найден, вывод сообщения и остановка работы
  //     while (1);
  // }

  pinMode(line_sensor_left, INPUT_PULLUP);
  pinMode(line_sensor_right, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);

  pinMode(pinE1, OUTPUT);
  pinMode(pinE2, OUTPUT);
  pinMode(pinH1, OUTPUT);
  pinMode(pinH2, OUTPUT);

  servo.attach(servo_pin);
  servo.write(null_deg_pos);

  pinMode(IR_sensor, INPUT);
  stop();

  for(;;) {
    if(digitalRead(11) == 0){
        break;
    } else {}
  }
  delay(1000);

  // for (;;) {
  //   Serial.println(getLuxSensor());
  //   delay(50);
  // }

    // delay(10);
    // Serial.println(analogRead(line_sensor_left));
    // delay(50);
    // line(47, 0);
    // drive(60, 60);


//   for (;;) {
//     drive(70, 70);
//   }
}




void loop() {

  drive_time(70, 300);

  for (int i=0; i < 6; i++) {
    line_distance_sensor(70);
    delay(delays);
    if (i == 5) {
      line_time(70, 20, 250);
    } else {
      line_time(70, 20, 350);
    }
    anti_turn(1, -50, 10);
    turn(1, -50, 100);
    delay(delays);
    line_per(60, 0, 1);
    delay(delays);

    
    drive_time(-70, 200);
    delay(delays);
    
    servo.write(up_deg_pos);
    delay(delays);

    for (;;) {
      if (!digitalRead(IR_sensor)) {
        drive_time(80, 400);
        drive_time(-70, 350);
        break;
      }
      
      if (analogRead(line_sensor_right) < threshold_level_1 && analogRead(line_sensor_left) < threshold_level_1) {
        stop();
        drive_time(-70, 200);
        break;
      }
      line(45, 0);
    } 
    
    drive_time(-70, 300);

    servo.write(down_deg_pos);
    delay(delays);

    for (;;) {
      if (!digitalRead(IR_sensor)) {
        drive_time(80, 400);
        drive_time(-70, 250);
        break;
      }
      
      if (analogRead(line_sensor_right) < threshold_level_1 && analogRead(line_sensor_left) < threshold_level_1) {
        stop();
        drive_time(-70, 200);
        break;
      }
      line(45, 0);
    } 

    drive_time(-70, 400);

    delay(delays);

    servo.write(null_deg_pos);
    turn(1, 50, 500);
    line_per(55, 0, 1);
    delay(delays);
    drive_time(70, 300);
    delay(delays);

    anti_turn(1, -55, 100);
    if (i == 5) {
      anti_turn(1, 50, 10);
    }
    

    line_time(70, 20, 400);
  }

  delay(delays);
  drive_time(65, 500);


  delay(1000000);

}