#include <Arduino.h>
#include <Servo.h>
#include <Ultrasonic.h>

#include <Wire.h>
#include <SPI.h>

Ultrasonic ultrasonic_sensor(12, 13);

Servo servo;
const int servo_pin = 9;

const int line_sensor_left = 14;
const int line_sensor_right = 15;

const int pinH1 = 7;  // Создаём константу указывая номер вывода H1 MotorShield (он управляет направлением 1 мотора)
const int pinE1 = 6;  // Создаём константу указывая номер вывода E1 MotorShield (он управляет скоростью    1 мотора)
const int pinE2 = 5;  // Создаём константу указывая номер вывода E2 MotorShield (он управляет скоростью    2 мотора)
const int pinH2 = 4;  // Создаём константу указывая номер вывода H2 MotorShield (он управляет направлением 2 мотора)

int threshold_level = 0;
int threshold_level_1 = 0;
int threshold_level_2 = 0;

int delays = 100;

int null_pos = 0;
int open_pos = 180;
int close_pos = 30;

int beep_pin = 2;

bool is_obj = 0;

int current = 1; // 0 - слева 1 - центр 2 - справа
int target = 0; // 0 - слева 1 - справа

void stop() {
  digitalWrite(pinH1, 0);
  digitalWrite(pinH2, 0);

  analogWrite(pinE1, 0);
  analogWrite(pinE2, 0);
}

// нулевое - 180 нижнее - 25, верхнее - 70

void drive(int speed_1, int speed_2) {

  bool direct_1 = false;
  bool direct_2 = false;

  if (speed_1 > 0) direct_1 = true;
  if (speed_2 > 0) direct_2 = true;

  digitalWrite(pinH1, direct_1);
  digitalWrite(pinH2, direct_2);

  analogWrite(pinE1, abs(speed_1)); 
  analogWrite(pinE2, abs(speed_2) * 1);

}


void drive_time(int speed, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, speed);
  }
  stop();
}

void drive_all_time(int speed1, int speed2, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed1, speed2);
  }
  stop();
}



void line(int speed = 30, int speed_slow = 0, int delay_=5) {

  bool line_sensor_left_value = false;
  bool line_sensor_right_value = false;

  if (digitalRead(line_sensor_left)) line_sensor_left_value = true;
  if (digitalRead(line_sensor_right)) line_sensor_right_value = true;

  Serial.print(line_sensor_left_value);
  Serial.print(" ");
  Serial.println(line_sensor_right_value);

  if (line_sensor_left_value == 0 && line_sensor_right_value == 0)
    drive(speed, speed);
  else if (line_sensor_left_value == 1 && line_sensor_right_value == 0) {
    drive(speed_slow, speed);
    delay(delay_);
  }
  else if (line_sensor_left_value == 0 && line_sensor_right_value == 1) {
    drive(speed, speed_slow);
    delay(delay_);
  } else {
    drive(speed, speed);
  }
}

void line_time(int speed, int speed_slow, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      line(speed, speed_slow);
  }
  stop();
}

void line_per(int speed, int speed_slow, int n) {
  for (int i = 0; i < n; i++) {
    for (;;) {
      if (digitalRead(line_sensor_right) and digitalRead(line_sensor_left)) {
        stop();
        break;
      }
      line(speed, speed_slow, 5);
    }
    delay(100);
    stop();
  }
}

void line_distance_sensor(int speed = 40, int speed_slow = 0, int dist = 40) {
  for (;;) {
    if (ultrasonic_sensor.read() < dist) {
      stop();
      break;
    }
    line(speed, speed_slow);
    delay(5);
  }
}



void turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    while ((!digitalRead(line_sensor_left) && speed < 0) || (!digitalRead(line_sensor_right) && speed > 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void anti_turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    while ((!digitalRead(line_sensor_left) && speed > 0) || (!digitalRead(line_sensor_right) && speed < 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void line_align(int speed=15) {
  for (int i=0; i<1; i++) {
    // drive_time(speed, 400);
    for (;;) {
      bool line_sensor_left_value = false;
      bool line_sensor_right_value = false;

      if (digitalRead(line_sensor_left)) line_sensor_left_value = true;
      if (digitalRead(line_sensor_right)) line_sensor_right_value = true;


      if (line_sensor_left_value == 0 && line_sensor_right_value == 0)
        drive(-speed, -speed);
      else if (line_sensor_left_value == 1 && line_sensor_right_value == 0) {
        drive(speed, -speed);
      }
      else if (line_sensor_left_value == 0 && line_sensor_right_value == 1) {
        drive(-speed, speed);
      } else {
        stop();
        break;
      }
    }
  }
}

bool turn_to_obj(int time_=500) {


  unsigned long millis_old = millis();

  for (;;) {
    drive(40, 0);
    if (ultrasonic_sensor.read() < 45) {
      stop();
      return 1;
    }
    if (millis() - millis_old > time_) {
      stop();
      return 0;
    }
  }


} 

void beep(int delay_) {
  tone(beep_pin, 1000, delay_);
}



void setup() {
  Serial.begin(9600);

  pinMode(line_sensor_left, INPUT_PULLUP);
  pinMode(line_sensor_right, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);

  pinMode(pinE1, OUTPUT);
  pinMode(pinE2, OUTPUT);
  pinMode(pinH1, OUTPUT);
  pinMode(pinH2, OUTPUT);

  pinMode(beep_pin, OUTPUT);

  servo.attach(servo_pin);
  servo.write(null_pos);
  stop();

  for(;;) {
    if(digitalRead(11) == 0){
        break;
    } else {}
  }
  delay(1000);


  // for(;;) {
  //   Serial.println(analogRead(line_sensor_right));
  // }

}

void transport() { // транспортировка - перевоз банки на противоположное положение
  drive_time(50, 350);
  servo.write(close_pos);
  drive_time(-70, 450);
  drive_all_time(-20, 20, 1300);
  line_align();
  drive_time(50, 100);

  for (;;) {
    // drive(35, 70);
    drive(30, 46);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_time(50, 250);
  line_align();

  drive_time(50, 350);
  for (;;) {
    // drive(35, 70);
    drive(30, 46);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }

  drive_all_time(35, 70, 300); // проезд к объекту
  line_align();
  
  drive_time(70, 250);
  turn(1, 25, 50);
  line_per(20, 0, 1);
  drive_time(50, 200);
  delay(500);
  servo.write(open_pos);
  drive_time(-50, 900);
  servo.write(null_pos);
}

void center_to_left() { // проезд к левой банке от центрального положения, транспортировка
  current = 2;
  drive_all_time(50, 50, 300);
  for (;;) {
    // drive(35, 70);
    drive(70, 40);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_all_time(70, 35, 300); // проезд к объекту
  line_align();
  drive_time(70, 250);
  servo.write(open_pos);
  turn(1, -25, 50);
  line_per(20, 0, 1);
  
  transport();

  is_obj = turn_to_obj();
  if (is_obj) {
    target = 1;
  } else {
    target = 0;
  }
  anti_turn(1, -40, 0);
}

void center_to_right() { // проезд к правой банке от центрального положения, транспортировка
  current = 0;
  drive_all_time(50, 50, 300);
  for (;;) {
    // drive(35, 70);
    drive(40, 70);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_all_time(35, 70, 300); // проезд к объекту
  line_align();
  drive_time(70, 250);
  servo.write(open_pos);
  turn(1, 25, 50);

  is_obj = turn_to_obj(700);
  if (is_obj) {
    target = 1;
  } else {
    target = 0;
  }
  anti_turn(1, -40, 0);
  line_per(20, 0, 1);

  transport();
}

void right_to_center() { // проезд к центру от правого положения
  current = 1;
  drive_all_time(-50, 50, 470);
  line_align();
  drive_time(50, 250);
  for (;;) {
    // drive(35, 70);
    drive(40, 70);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_time(50, 250);
  line_align();
}

void left_to_center() { // проезд к центру от левого положения
  current = 1;
  drive_all_time(50, -50, 470);
  line_align();
  drive_time(50, 250);
  for (;;) {
    // drive(35, 70);
    drive(70, 40);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_time(50, 250);
  line_align();

}

void right_to_right() { // проезд к правой банке от правого положения, транспортировка
  current = 0;
  drive_all_time(-50, 50, 470);
  line_align();
  drive_time(50, 250);
  for (;;) {
    // drive(35, 70);
    drive(60, 60);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_time(50, 250);
  line_align();
  drive_time(50, 150);
  servo.write(open_pos);
  turn(1, 25, 50);
  
  is_obj = turn_to_obj();
  if (is_obj) {
    target = 1;
  } else {
    target = 0;
  }
  anti_turn(1, -40, 0);
  line_per(20, 0, 1);

  transport();
}

void left_to_left() { // проезд к левой банке от левого положения, транспортировка
  current = 2;
  drive_all_time(50, -50, 470);
  line_align();
  drive_time(50, 250);
  for (;;) {
    // drive(35, 70);
    drive(60, 60);
    if (digitalRead(line_sensor_left)) {
      stop();
      break;
    }
  }
  drive_time(50, 250);
  line_align();
  drive_time(50, 150);
  servo.write(open_pos);
  turn(1, -25, 50);
  line_per(20, 0, 1);

  transport();
  
  is_obj = turn_to_obj();
  if (is_obj) {
    target = 1;
  } else {
    target = 0;
  }
  anti_turn(1, -40, 0);
}

void right_finish() {
  right_to_center();
  drive_time(40, 200);
  turn(1, 30, 50);
  line_per(50, 10, 1);
  drive_all_time(45, 40, 400);
}

void left_finish() {
  left_to_center();
  drive_time(40, 200);
  turn(1, -30, 50);
  line_per(50, 10, 1);
  drive_all_time(45, 40, 400);
}

void loop() {
  // left_to_left();

  line_time(70, 0, 350);
  is_obj = turn_to_obj();
  turn(1, -30, 0);

  if (is_obj) {
    target = 0;
    drive_all_time(-50, 50, 470);
    line_align();
  } else {
    target = 1;
    drive_all_time(50, -50, 470);
    line_align();
  }



  for (int i=0; i<3; i++) {
    if (current == 1 && target == 0) {
      center_to_left();
    } else if (current == 1 && target == 1) {
      center_to_right();
    } else if (current == 0 && target == 1) {
      left_to_center();
      center_to_right();
    } else if (current == 2 && target == 0) {
      right_to_center();
      center_to_left();
    } else if (current == 0 && target == 0) {
      left_to_left();
    } else if (current == 2 && target == 1) {
      right_to_right();
    }
  }
  
  if (current == 2) {
    right_finish();
  } else if (current == 0) {
    left_finish();
  }
  

  // ------------------------------------------


  // drive_all_time(50, 50, 300);
  // for (;;) {
  //   // drive(35, 70);
  //   drive(40, 70);
  //   if (analogRead(line_sensor_left) < threshold_level) {
  //     stop();
  //     break;
  //   }
  // }
  // drive_all_time(35, 70, 300);
  // line_align();

  // drive_time(70, 200);

  // servo.write(open_pos);
  // turn(1, 30, 50);

  // line_per(20, 0, 1);


  // // // если есть
  // // drive_all_time(-50, 50, 500); // об этом мы пожалеем
  // turn(1, -35, 10);
  // line_per(20, 0, 1);
  // drive_all_time(30, 45, 450);
  // servo.write(close_pos);
  // drive_time(70, 350);

  // drive_all_time(-50, 50, 430);
  // line_align();
  // drive_time(70, 400);
  // drive_all_time(-50, 50, 400);

  // for (;;) {
  //   drive(50, 50);
  //   if (analogRead(line_sensor_left) < threshold_level_2 && analogRead(line_sensor_right) < threshold_level_2) {
  //     stop();
  //     break;
  //   }
  // }
  // drive_time(70, 150);
  // line_align();


  




  // // ----------------------------------------------
  // drive_time(-70, 300);
  // drive_all_time(50, -50, 430);
  // line_align();



  // drive_time(70, 450);
  // drive_all_time(50, -50, 430);
  // drive_time(70, 600);
  // line_align();



  




  


  delay(1000000);

}