#include <Arduino.h>
#include <Servo.h>
#include <Ultrasonic.h>

#include <Wire.h>
#include <SPI.h>

Ultrasonic ultrasonic_sensor(12, 13);

Servo servo;
const int servo_pin = 9;

const int line_sensor_left = 14;
const int line_sensor_right = 15;

const int pinH1 = 7;  // Создаём константу указывая номер вывода H1 MotorShield (он управляет направлением 1 мотора)
const int pinE1 = 6;  // Создаём константу указывая номер вывода E1 MotorShield (он управляет скоростью    1 мотора)
const int pinE2 = 5;  // Создаём константу указывая номер вывода E2 MotorShield (он управляет скоростью    2 мотора)
const int pinH2 = 4;  // Создаём константу указывая номер вывода H2 MotorShield (он управляет направлением 2 мотора)

int threshold_level = 0;
int threshold_level_1 = 0;
int threshold_level_2 = 0;

int delays = 100;

int null_pos = 100;
int shot_pos = 0;

int beep_pin = 2;

bool is_obj = 0;

int current_state = 1; // -1 - лево 0 - центр 1 - право
int current_count_lines = 0;


void stop() {
  digitalWrite(pinH1, 0);
  digitalWrite(pinH2, 0);

  analogWrite(pinE1, 0);
  analogWrite(pinE2, 0);
}

// нулевое - 180 нижнее - 25, верхнее - 70

void drive(int speed_1, int speed_2) {

  bool direct_1 = false;
  bool direct_2 = false;

  if (speed_1 > 0) direct_1 = true;
  if (speed_2 > 0) direct_2 = true;

  digitalWrite(pinH1, direct_1);
  digitalWrite(pinH2, direct_2);

  analogWrite(pinE1, abs(speed_1)); 
  analogWrite(pinE2, abs(speed_2) * 1);

}


void drive_time(int speed, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, speed);
  }
  stop();
}

void drive_all_time(int speed1, int speed2, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed1, speed2);
  }
  stop();
}



void line(int speed = 30, int speed_slow = 0, int delay_=5) {

  bool line_sensor_left_value = false;
  bool line_sensor_right_value = false;

  if (digitalRead(line_sensor_left)) line_sensor_left_value = true;
  if (digitalRead(line_sensor_right)) line_sensor_right_value = true;

  Serial.print(line_sensor_left_value);
  Serial.print(" ");
  Serial.println(line_sensor_right_value);

  if (line_sensor_left_value == 0 && line_sensor_right_value == 0)
    drive(speed, speed);
  else if (line_sensor_left_value == 1 && line_sensor_right_value == 0) {
    drive(speed_slow, speed);
    delay(delay_);
  }
  else if (line_sensor_left_value == 0 && line_sensor_right_value == 1) {
    drive(speed, speed_slow);
    delay(delay_);
  } else {
    drive(speed, speed);
  }
}

void line_time(int speed, int speed_slow, int time_) {
  unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      line(speed, speed_slow);
  }
  stop();
}

void line_per(int speed=50, int speed_slow=0, int n=1) {
  for (int i = 0; i < n; i++) {
    for (;;) {
      if (digitalRead(line_sensor_right) and digitalRead(line_sensor_left)) {
        stop();
        break;
      }
      line(speed, speed_slow, 5);
    }
    delay(100);
    stop();
  }
}

void to_line(int state=1) { // 1 - черн 0- белая

  for (;;) {
    if (digitalRead(line_sensor_right) == state and digitalRead(line_sensor_left) == state) {
      stop();
      break;
    }
    line(40, 0, 5);
  }
  delay(100);
  stop();

}

void line_distance_sensor(int speed = 40, int speed_slow = 0, int dist = 40) {
  for (;;) {
    if (ultrasonic_sensor.read() < dist) {
      stop();
      break;
    }
    line(speed, speed_slow);
    delay(5);
  }
}



void turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    while ((!digitalRead(line_sensor_left) && speed < 0) || (!digitalRead(line_sensor_right) && speed > 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void anti_turn(int angle, int speed, int time_) {
  for (int i = 0; i < angle; i++) {
    unsigned long millis_old = millis();
    while (millis() - millis_old < time_) {
      drive(speed, -speed);
    }

    while ((!digitalRead(line_sensor_left) && speed > 0) || (!digitalRead(line_sensor_right) && speed < 0)) {
      drive(speed, -speed);
    }
    stop();
  }
}

void line_align(int speed=15) {
  for (int i=0; i<1; i++) {
    // drive_time(speed, 400);
    for (;;) {
      bool line_sensor_left_value = false;
      bool line_sensor_right_value = false;

      if (digitalRead(line_sensor_left)) line_sensor_left_value = true;
      if (digitalRead(line_sensor_right)) line_sensor_right_value = true;


      if (line_sensor_left_value == 0 && line_sensor_right_value == 0)
        drive(-speed, -speed);
      else if (line_sensor_left_value == 1 && line_sensor_right_value == 0) {
        drive(speed, -speed);
      }
      else if (line_sensor_left_value == 0 && line_sensor_right_value == 1) {
        drive(-speed, speed);
      } else {
        stop();
        break;
      }
    }
  }
}

bool turn_to_obj(int time_=500) {


  unsigned long millis_old = millis();

  for (;;) {
    drive(40, 0);
    if (ultrasonic_sensor.read() < 45) {
      stop();
      return 1;
    }
    if (millis() - millis_old > time_) {
      stop();
      return 0;
    }
  }


} 

void beep(int delay_) {
  tone(beep_pin, 1000, delay_);
}



void setup() {
  Serial.begin(9600);

  pinMode(line_sensor_left, INPUT_PULLUP);
  pinMode(line_sensor_right, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);

  pinMode(pinE1, OUTPUT);
  pinMode(pinE2, OUTPUT);
  pinMode(pinH1, OUTPUT);
  pinMode(pinH2, OUTPUT);

  pinMode(beep_pin, OUTPUT);

  servo.attach(servo_pin);
  servo.write(null_pos);
  stop();

  for(;;) {
    if(digitalRead(11) == 0){
        break;
    } else {}
  }
  delay(1000);


  // for(;;) {
  //   Serial.println(analogRead(line_sensor_right));
  // }

}

void count() {
  drive_all_time(50, 60, 200);
  line_align();
  drive_all_time(50, 60, 700);
}



void loop() {

  // for(;;) {
  //     unsigned long millis_old = millis();

  //     to_line();

  //     if (millis() - millis_old > 100) {
  //       break;
  //     }


  //     to_line(1);


  //   }

  for (int i=0; i<3; i++) {
    line_per(50, 0, 1);
    drive_time(50, 400);


    turn(1, 50 * current_state, 200);
    line_per(50, 0, 1);
    // drive_time(50, 200);
    // line_align();

    if (current_state != 0) {
      drive_time(50, 300);
      turn(1, -50 * current_state, 200);
      line_per(50, 0, 1);

    }
    count();

    line_per(50, 0, 1);

    drive_time(50, 300);

    line_align();

    drive_all_time(50, 90, 200);

    delay(1000);
    servo.write(shot_pos);
    delay(1000);

    if (i==1) {
      drive_all_time(-50, -90, 230);
    } else {
      drive_all_time(-50, -90, 400);
    }

    

    turn(1, 50, 200);

    servo.write(null_pos);

    line_per(50, 0, 1);

    count();

    line_per(50, 0, 1);
    // drive_time(50, 200);
    // line_align();

    if (current_state != 0) {
      drive_time(50, 300);
      turn(1, 50 * current_state, 200);
      line_per(50, 0, 1);

    }

    drive_time(50, 500);
    anti_turn(1, -50 * current_state, 50);
    turn(1, -50 * current_state, 200);
    line_per(50, 0, 1);
    drive_time(50, 450);
    if (i != 2) {
      beep(300);
      delay(10000);
      beep(3000);
      delay(3000);
      drive_time(-50, 700);
      turn(1, 50, 200);
    }
    

    current_state -= 1;



  }


  delay(1000000);

}